/* GDK - The GIMP Drawing Kit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
 */

#include "config.h"

#include "gdkdirectfb.h"
#include "gdkprivate-directfb.h"


void
_gdk_windowing_window_get_offsets (GdkWindow *window,
                                   gint      *x_offset,
                                   gint      *y_offset)
{
  if (x_offset) 
    *x_offset = 0;
  if (y_offset)
    *y_offset = 0;
}

gboolean
_gdk_windowing_window_queue_antiexpose (GdkWindow *window,
                                        GdkRegion *area)
{
  return FALSE;
}

/**
 * gdk_window_scroll:
 * @window: a #GdkWindow
 * @dx: Amount to scroll in the X direction
 * @dy: Amount to scroll in the Y direction
 *
 * Scroll the contents of its window, both pixels and children, by
 * the given amount. Portions of the window that the scroll operation
 * brings in from offscreen areas are invalidated. The invalidated
 * region may be bigger than what would strictly be necessary.
 * (For X11, a minimum area will be invalidated if the window has
 * no subwindows, or if the edges of the window's parent do not extend
 * beyond the edges of the window. In other cases, a multi-step process
 * is used to scroll the window which may produce temporary visual
 * artifacts and unnecessary invalidations.)
 **/
#if 0
void
gdk_window_scroll (GdkWindow *window,
                   gint       dx,
                   gint       dy)
{
  GdkWindowObject       *private;
  GdkDrawableImplDirectFB *impl;

  g_return_if_fail (GDK_IS_WINDOW (window));

  private = GDK_WINDOW_OBJECT (window);
  impl = GDK_DRAWABLE_IMPL_DIRECTFB (private->impl);

  if (impl->surface) {
    DFBRegion region = { dx, dy, impl->width, impl->height };
    
    impl->surface->Blit( impl->surface, impl->surface, NULL, dx, dy );
    
    gdk_directfb_add_update (impl, &region);
  }

  {
    GList *tmp_list;
    GdkRectangle org_rect = { 0, 0, impl->width, impl->height };
    GdkRectangle dest_rect = { dx, dy, impl->width, impl->height };

    gdk_rectangle_intersect (&dest_rect, &org_rect, &dest_rect);

    gdk_window_invalidate_rect_clear (window, &dest_rect, TRUE);
/*
      tmp_list = obj->children;
      while (tmp_list)
         {
            GdkWindow * child = GDK_WINDOW (tmp_list->data);

            gdk_window_move (child, obj->x + dx, obj->y + dy);

            tmp_list = tmp_list->next;
         }*/
  }
}
#endif



void
gdk_window_scroll (GdkWindow *window,
                   gint       dx,
                   gint       dy)
{
  GdkWindowObject         *private;
  GdkDrawableImplDirectFB *impl;
  GdkRegion               *invalidate_region;
  GdkRectangle             dest_rect;
  GdkRectangle             clip_rect;
  GList                   *tmp_list;

  g_return_if_fail (GDK_IS_WINDOW (window));

  private = GDK_WINDOW_OBJECT (window);
  impl = GDK_DRAWABLE_IMPL_DIRECTFB (private->impl);
  
  clip_rect.x = 0;
  clip_rect.y = 0;
  clip_rect.width  = impl->width;
  clip_rect.height = impl->height;

  invalidate_region = gdk_region_rectangle (&clip_rect);
      
  dest_rect = clip_rect;
  dest_rect.x += dx;
  dest_rect.y += dy;
  gdk_rectangle_intersect (&dest_rect, &clip_rect, &dest_rect);

  if (dest_rect.width > 0 && dest_rect.height > 0)
    {
      GdkRegion *tmp_region;
      
      tmp_region = gdk_region_rectangle (&dest_rect);
      gdk_region_subtract (invalidate_region, tmp_region);
      gdk_region_destroy (tmp_region);

      if (impl->surface) 
        {
          DFBRegion region = { dx, dy, impl->width, impl->height };
          
          impl->surface->Blit( impl->surface, impl->surface, NULL, dx, dy );
          
          gdk_directfb_add_update (impl, &region);
        }
    }
  
  gdk_window_invalidate_region_clear (window, invalidate_region);
  gdk_region_destroy (invalidate_region);

  for (tmp_list = private->children; tmp_list; tmp_list = tmp_list->next)
    {
      GdkWindowObject         *obj      = GDK_WINDOW_OBJECT (tmp_list->data);
      GdkDrawableImplDirectFB *obj_impl = GDK_DRAWABLE_IMPL_DIRECTFB (obj->impl);

      gdk_directfb_move_resize_child (tmp_list->data,
                                      obj->x + dx,
                                      obj->y + dy,
                                      obj_impl->width,
                                      obj_impl->height);
    }
}
